package com.livelike.flutter.livelike_flutter_sdk

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.view.View
import com.google.gson.Gson
import com.google.gson.JsonParser
import com.livelike.engagementsdk.LiveLikeContentSession
import com.livelike.engagementsdk.core.services.messaging.proxies.LiveLikeWidgetEntity
import com.livelike.engagementsdk.core.services.messaging.proxies.WidgetInterceptor
import com.livelike.engagementsdk.core.services.messaging.proxies.WidgetLifeCycleEventsListener
import com.livelike.engagementsdk.widget.view.WidgetView
import com.livelike.engagementsdk.widget.viewModel.WidgetStates
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import org.json.JSONObject

class WidgetViewFactory(
    private val plugin: LivelikeFlutterSdkPlugin,
    private val messenger: BinaryMessenger
) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {
    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        val creationParams = args as Map<String?, Any?>?
        return FlutterPluginWidgetView(context, plugin, viewId, creationParams, messenger)
    }
}

class FlutterPluginWidgetView(
    context: Context?,
    var plugin: LivelikeFlutterSdkPlugin,
    viewId: Int,
    creationParams: Map<String?, Any?>?,
    messenger: BinaryMessenger
) : PlatformView, MethodChannel.MethodCallHandler {
    private var pluginWidgetView: PluginWidgetView
    private var widgetView: WidgetView
    private val methodChannel: MethodChannel = MethodChannel(messenger, "widgetView$viewId")
    private val uiThreadHandler: Handler = Handler(Looper.getMainLooper())
    private var currentSession: LiveLikeContentSession? = null

    init {
        methodChannel.setMethodCallHandler(this)
        pluginWidgetView = PluginWidgetView(context!!)
        pluginWidgetView.findViewById<WidgetView>(R.id.plugin_widget_view)
        widgetView = pluginWidgetView.findViewById(R.id.plugin_widget_view)
    }

    override fun getView(): View {
        return pluginWidgetView
    }

    override fun dispose() {
        currentSession?.widgetStream?.unsubscribe(this)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "setSession" -> {
                val programId = call.argument<String>("programId") as String
                plugin.sessionMap[programId]?.let { session ->
                    widgetView.setSession(session)
                    this.currentSession = session
                    session.widgetInterceptor = object : WidgetInterceptor() {
                        override fun widgetWantsToShow(widgetData: LiveLikeWidgetEntity) {
                            uiThreadHandler.post {
                                plugin.widgetStreamHandler.eventSink?.success(
                                    hashMapOf(
                                        "show" to true,
                                        "data" to toMap(JSONObject(Gson().toJson(widgetData)))
                                    )
                                )
                                uiThreadHandler.postDelayed({
                                    showWidget()
                                }, 500L)
                            }
                        }
                    }
                    widgetView.widgetLifeCycleEventsListener =
                        object : WidgetLifeCycleEventsListener() {
                            override fun onUserInteract(widgetData: LiveLikeWidgetEntity) {

                            }

                            override fun onWidgetDismissed(widgetData: LiveLikeWidgetEntity) {
                                uiThreadHandler.post {
                                    plugin.widgetStreamHandler.eventSink?.success(
                                        hashMapOf("show" to false)
                                    )
                                }
                            }

                            override fun onWidgetInteractionCompleted(widgetData: LiveLikeWidgetEntity) {

                            }

                            override fun onWidgetPresented(widgetData: LiveLikeWidgetEntity) {

                            }

                            override fun onWidgetStateChange(
                                state: WidgetStates,
                                widgetData: LiveLikeWidgetEntity
                            ) {

                            }
                        }
                }
                result.success(null)
            }
            "displayWidget" -> {
                widgetView.displayWidget(
                    plugin.engagementSDK,
                    JsonParser.parseString(call.arguments.toString()).asJsonObject
                )
                result.success(null)
            }
        }
    }
}