import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:http/http.dart' as http;
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';

import 'home.dart';

class SelectChannel extends StatelessWidget {
  final controller = Get.put(SelectChannelController(), permanent: true);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        actions: [
          Obx(() =>
              controller.previousUrl.value != null && !controller.loading.value
                  ? TextButton(
                      onPressed: () {
                        controller.loadData(url: controller.previousUrl.value);
                      },
                      child: Text(
                        "Previous",
                        style: TextStyle(color: Colors.white),
                      ))
                  : SizedBox.shrink()),
          Obx(() =>
              controller.nextUrl.value != null && !controller.loading.value
                  ? TextButton(
                      onPressed: () {
                        controller.loadData(url: controller.nextUrl.value);
                      },
                      child: Text(
                        "Next",
                        style: TextStyle(color: Colors.white),
                      ))
                  : SizedBox.shrink()),
        ],
      ),
      body: Obx(() => controller.loading.value
          ? Center(
              child: CircularProgressIndicator(),
            )
          : ListView.builder(
              itemCount: controller.channels.length,
              itemBuilder: (context, index) {
                final channel = controller.channels[index];
                return ListTile(
                  onTap: () {
                    Get.back(result: channel);
                  },
                  title: Text("${channel.name}"),
                  subtitle: Text("${channel.id}"),
                );
              },
            )),
    );
  }
}

class SelectChannelController extends GetxController {
  final channels = <Channel>[].obs;
  final nextUrl = Rx<String?>(null);
  final previousUrl = Rx<String?>(null);
  final loading = false.obs;

  @override
  void onInit() {
    super.onInit();
    if (channels.isEmpty) loadData();
  }

  void loadData({String? url}) async {
    loading.value = true;
    final sdk = Get.find<EngagementSDK>();
    final response = await http.get(Uri.parse(url ??
        "https://cf-blast.livelikecdn.com/api/v1/programs/?client_id=${sdk.clientId}"));
    final Map<String, dynamic> data = jsonDecode(response.body);
    nextUrl.value = data["next"];
    previousUrl.value = data["previous"];
    final List result = data["results"];
    channels
      ..clear()
      ..addAll(result.map((e) {
        final channel = Channel();
        channel.name = e["title"];
        channel.id = e["id"];
        channel.stream_url = e["stream_url"];
        return channel;
      }).toList());
    loading.value = false;
  }
}
