import 'dart:convert';

import 'package:faker/faker.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:http/http.dart' as http;
import 'package:livelike_flutter_sdk/livelike_flutter_sdk.dart';
import 'package:livelike_flutter_sdk_example/home.dart';

class WidgetsOnlyScreen extends StatelessWidget {
  final controller = Get.put(WidgetsOnlyController());

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        actions: [
          Obx(() => controller.loading.value
              ? Center(
                  child: SizedBox(
                      width: 20,
                      height: 20,
                      child: CircularProgressIndicator(
                        strokeWidth: 1,
                        valueColor: AlwaysStoppedAnimation<Color>(Colors.white),
                      )),
                )
              : SizedBox.shrink()),
          Padding(padding: const EdgeInsets.all(5)),
          Obx(() => DropdownButtonHideUnderline(
                  child: DropdownButton<String>(
                value: controller.selectedTime.value,
                items: controller.times.entries
                    .map((e) => DropdownMenuItem<String>(
                          child: Text(
                            "${e.value}",
                            // style: TextStyle(color: Colors.white),
                          ),
                          value: e.key,
                        ))
                    .toList(),
                dropdownColor: Colors.white,
                onChanged: (value) {
                  controller.selectedTime.value = value!;
                },
              ))),
        ],
      ),
      body: Column(
        children: [
          Expanded(
            child: Obx(() => controller.session.value != null
                ? Padding(
                    padding: const EdgeInsets.all(10),
                    child: WidgetView(
                      session: controller.session.value!,
                      key: Key(controller.session.value!.programId!),
                    ),
                  )
                : SizedBox.shrink()),
          ),
          Expanded(
            child: ListView.builder(
              itemCount: controller.listItems.length,
              itemBuilder: (context, index) {
                final item = controller.listItems[index];
                return ListTile(
                  title: Text(
                    "${item.title}",
                    style: TextStyle(
                        fontWeight:
                            item.isHeader ? FontWeight.bold : FontWeight.w300),
                  ),
                  onTap: item.isHeader
                      ? null
                      : () {
                          controller.performApi(item);
                        },
                );
              },
            ),
          )
        ],
      ),
    );
  }
}

class WidgetsOnlyController extends GetxController with WidgetsBindingObserver {
  final Rx<Session?> session = Rx(null);
  final sdk = Get.find<EngagementSDK>();
  final _images = [
    "https://cf-blast-storage-staging.livelikecdn.com/assets/e38f0089-00be-4236-830a-6989e8298b50.jpg",
    "https://cf-blast-storage-staging.livelikecdn.com/assets/3c4db954-216b-40a8-ac34-9d43036b009d.jpg",
    "https://cf-blast-storage-staging.livelikecdn.com/assets/d223c74b-c29c-44a0-a3ce-1b8301971cf3.jpg",
    "https://cf-blast-storage-staging.livelikecdn.com/assets/84af0879-cfe9-4e37-8f38-adfe0a51ee57.jpg",
    "https://cf-blast-storage-staging.livelikecdn.com/assets/0cd1f9ad-c92c-4f6d-ada5-bab926df175f.jpg"
  ];
  final _alertLinks = [
    "tel:8372873843432",
    "https://www.google.co.in",
    "https://www.youtube.com/watch?v=m5C6gKviWIQ",
    "nothing://nothing"
  ];
  final listItems = <PostType>[
    PostType.header("Text Poll"),
    PostType.body("2 Options", "text-polls", 2, WidgetType.textPoll),
    PostType.body("4 Options", "text-polls", 4, WidgetType.textPoll),
    PostType.header("Image Poll"),
    PostType.body("2 Options", "image-polls", 2, WidgetType.imagePoll),
    PostType.body("4 Options", "image-polls", 4, WidgetType.imagePoll),
    PostType.header("Text Quiz"),
    PostType.body("2 Options", "text-quizzes", 2, WidgetType.textQuiz),
    PostType.body("4 Options", "text-quizzes", 4, WidgetType.textQuiz),
    PostType.header("Image Quiz"),
    PostType.body("2 Options", "image-quizzes", 2, WidgetType.imageQuiz),
    PostType.body("4 Options", "image-quizzes", 4, WidgetType.imageQuiz),
    PostType.header("Text Prediction"),
    PostType.body(
        "2 Options", "text-predictions", 2, WidgetType.textPrediction),
    PostType.body(
        "4 Options", "text-predictions", 4, WidgetType.textPrediction),
    PostType.header("Image Prediction"),
    PostType.body(
        "2 Options", "image-predictions", 2, WidgetType.imagePrediction),
    PostType.body(
        "4 Options", "image-predictions", 4, WidgetType.imagePrediction),
  ];

  final loading = false.obs;
  final times = {"PT10S": "10 sec", "PT20S": "20 sec", "PT30S": "30 sec"};
  final selectedTime = "PT10S".obs;

  @override
  void onInit() async {
    super.onInit();
    WidgetsBinding.instance?.addObserver(this);
    final Channel channel = Get.arguments;
    loading.value = true;
    session.value = await sdk.createSession(channel.id!);
    loading.value = false;
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    switch (state) {
      case AppLifecycleState.resumed:
        session.value?.resume();
        break;
      case AppLifecycleState.paused:
        session.value?.pause();
        break;
      case AppLifecycleState.inactive:
        break;
      case AppLifecycleState.detached:
        break;
    }
  }

  @override
  void dispose() {
    super.dispose();
    WidgetsBinding.instance?.removeObserver(this);
  }

  void performApi(PostType postType) async {
    final options = [];
    for (int i = 1; i <= postType.count; i++) {
      switch (postType.widgetType) {
        case WidgetType.textPoll:
          options.add({"description": faker.lorem.word()});
          break;
        case WidgetType.textPrediction:
        case WidgetType.textQuiz:
          options
              .add({"is_correct": i == 1, "description": faker.lorem.word()});
          break;
        case WidgetType.imagePoll:
        case WidgetType.imageSlider:
        case WidgetType.cheerMeter:
          options.add({
            "description": faker.lorem.word(),
            "image_url": faker.randomGenerator.element(_images)
          });
          break;
        case WidgetType.imagePrediction:
        case WidgetType.imageQuiz:
          options.add({
            "description": faker.lorem.word(),
            "image_url": faker.randomGenerator.element(_images),
            "is_correct": i == 1
          });
          break;
        default:
      }
    }

    Get.log("OPTIONS: ${options.length}");

    var body = {
      "question": faker.lorem.word(),
      "program_id": session.value!.programId!,
      "timeout": selectedTime.value,
      (postType.widgetType == WidgetType.imageQuiz ||
              postType.widgetType == WidgetType.textQuiz
          ? "choices"
          : "options"): options,
    };

    if (postType.widgetType == WidgetType.alert) {
      switch (postType.count) {
        case 1:
          body = {
            "title": "Alert",
            "text": faker.lorem.word(),
            "program_id": session.value!.programId!,
            "timeout": selectedTime.value,
          };
          break;
        case 2:
          body = {
            "title": "Alert",
            "image_url": faker.randomGenerator.element(_images),
            "program_id": session.value!.programId!,
            "timeout": selectedTime.value,
          };
          break;
        case 3:
          body = {
            "title": "Alert",
            "text": faker.lorem.word(),
            "image_url": faker.randomGenerator.element(_images),
            "program_id": session.value!.programId!,
            "timeout": selectedTime.value,
          };
          break;
        case 4:
          body = {
            "title": "Alert",
            "text": faker.lorem.word(),
            "image_url": faker.randomGenerator.element(_images),
            "program_id": session.value!.programId!,
            "timeout": selectedTime.value,
            "link_url": faker.randomGenerator.element(_alertLinks),
            "link_label": faker.lorem.word(),
          };
          break;
        default:
      }
    }
    Get.log("REQUEST: ${jsonEncode(body)}");
    loading.value = true;
    final response = await http.post(
        Uri.parse("https://cf-blast.livelikecdn.com/api/v1/${postType.url}/"),
        headers: {
          "Authorization":
              "Bearer zslM9_lbiy3SWkMbKsoGfAkK2Kg46dfVkN1Zsdt8K_P3BJU-AJOeSQ",
          "Content-Type": "application/json"
        },
        body: jsonEncode(body));
    final data = jsonDecode(response.body);
    final scheduleUrl = data["schedule_url"];
    await http.put(Uri.parse(scheduleUrl), headers: {
      "Authorization":
          "Bearer zslM9_lbiy3SWkMbKsoGfAkK2Kg46dfVkN1Zsdt8K_P3BJU-AJOeSQ",
      "Content-Type": "application/json"
    });
    loading.value = false;
    if (postType.widgetType == WidgetType.textPrediction ||
        postType.widgetType == WidgetType.imagePrediction) {
      final scheduleData = jsonDecode(response.body);
      final List followUps = scheduleData["follow_ups"];
      final followUp = followUps[0];
      final List<Map> predictionOption = List.from(data["options"]);
      for (int i = 0; i < predictionOption.length; i++) {
        Map d = predictionOption[i];
        if (i == 0) {
          d["vote_count"] = 0;
          d["is_correct"] = true;
        } else if (i == 1) {
          d["vote_count"] = 1;
          d["is_correct"] = false;
        }
        predictionOption[i] = d;
      }
      await Future.delayed(const Duration(seconds: 15));
      Get.log("Schedule Url: ${followUp["url"]}");
      final followUpResponse = await http.patch(Uri.parse(followUp["url"]),
          headers: {
            "Authorization":
                "Bearer zslM9_lbiy3SWkMbKsoGfAkK2Kg46dfVkN1Zsdt8K_P3BJU-AJOeSQ",
            "Content-Type": "application/json"
          },
          body: jsonEncode({
            "program_date_time": scheduleData["program_date_time"],
            "question": scheduleData["question"],
            "scheduled_at": scheduleData["scheduled_at"],
            "timeout": "P0DT00H00M07S",
            "options": predictionOption
          }));
      final followUpResponseData = jsonDecode(followUpResponse.body);
      await http.put(Uri.parse(followUpResponseData["schedule_url"]), headers: {
        "Authorization":
            "Bearer zslM9_lbiy3SWkMbKsoGfAkK2Kg46dfVkN1Zsdt8K_P3BJU-AJOeSQ",
        "Content-Type": "application/json"
      });
    }
  }
}

class PostType {
  final String title;
  bool isHeader = false;
  String? url;
  int count = 0;
  WidgetType? widgetType;

  PostType.header(this.title) {
    isHeader = true;
  }

  PostType.body(this.title, this.url, this.count, this.widgetType) {
    isHeader = false;
  }
}

enum WidgetType {
  textPoll,
  textQuiz,
  cheerMeter,
  imagePoll,
  imageQuiz,
  textPrediction,
  imagePrediction,
  imageSlider,
  alert
}
